/*
 * r.recolor -
 * developed from r.rescale.eq by
 * Timur Kusherbaev 
 * SJ Geophysics Ltd. (03/04/02)
 */

#include <string.h>
#include "gis.h"
#include "local_proto.h"

static FILE *fd;

static FILE *fsteps;

int main (int argc, char *argv[])
{
    char buf[512];
    DCELL old_min, old_max;
    DCELL new_min, new_max;
    long nlev;
    int typ;
    int cat;
    struct Cell_stats statf;
    char *old_name;
    char *mapset;
    char mt[16];
    char *meth;
    char fsn[256];
    char *fsname;
    char *me;
    double step;
    int palette;
    
    struct
        {
        struct Option *input, *from1, *from2, *output, *count, *method, *type,
	*step, *palette, *file;
    } parm;
    struct
        {
        struct Flag *quiet;
    } flag;
    struct GModule *module;

    G_gisinit (argv[0]);
    
    /* Set description */
    module              = G_define_module();
    module->description = ""\
    "Build color distribution for raster map categories";

    /* Define the different options */

    parm.input = G_define_option() ;
    parm.input->key        = "input";
    parm.input->type       = TYPE_STRING;
    parm.input->required   = YES;
    parm.input->gisprompt  = "old,cell,raster" ;
    parm.input->description= "The name of the raster map to build color bar" ;

    parm.from1 = G_define_option() ;
    parm.from1->key        = "from";
    parm.from1->key_desc   = "min";
    parm.from1->type       = TYPE_DOUBLE;
    parm.from1->required   = NO;
    parm.from1->description= "Min data range to be used (default: min data range)";

    parm.from2 = G_define_option() ;
    parm.from2->key        = "to";
    parm.from2->key_desc   = "max";
    parm.from2->type       = TYPE_DOUBLE;
    parm.from2->required   = NO;
    parm.from2->description= "Max data range to be used (default: max data range)";

    parm.count = G_define_option() ;
    parm.count->key        = "count";
    parm.count->key_desc   = "nlev";
    parm.count->type       = TYPE_INTEGER;
    parm.count->required   = NO;
    parm.count->description= "Count of levels";

    parm.method = G_define_option() ;
    parm.method->key        = "method";
    parm.method->key_desc   = "area|line";
    parm.method->type       = TYPE_STRING;
    parm.method->required   = NO;
    parm.method->description= "Method of color generation area-based on equal area, line-linear" ;

    parm.type = G_define_option() ;
    parm.type->key        = "type";
    parm.type->key_desc   = "0|1";
    parm.type->type       = TYPE_INTEGER;
    parm.type->required   = NO;
    parm.type->description= "Type of map: 0 - image | 1 - contour";
    
    parm.step = G_define_option() ;
    parm.step->key        = "step";
    parm.step->key_desc   = "value ";
    parm.step->type       = TYPE_DOUBLE;
    parm.step->required   = NO;
    parm.step->description= "Step of labels - works only for linear color generation";

    parm.palette = G_define_option() ;
    parm.palette->key        = "palette";
    parm.palette->key_desc   = "0|1|2|3|4|5 or 10|11|12|13 - one of values";
    parm.palette->type       = TYPE_INTEGER;
    parm.palette->required   = NO;
    parm.palette->description= "Palette for color bar generation" ;

    parm.file = G_define_option() ;
    parm.file->key        = "file";
    parm.file->key_desc   = "file name for levels";
    parm.file->type       = TYPE_STRING;
    parm.file->required   = NO;
    parm.file->description= "Name of file which contain steps for levels" ;


    flag.quiet = G_define_flag();
    flag.quiet->key = 'q';
    flag.quiet->description = "Quietly";

//    G_disable_interactive();
    if (G_parser(argc, argv))
        exit(-1);

    old_name = parm.input->answer;

    mapset = G_find_cell (old_name,"");
    if (mapset == NULL)
    {
        sprintf (buf, "%s - not found\n", old_name);
        G_fatal_error (buf);
        exit(1);
    }

    if (parm.from1->answer)
        sscanf (parm.from1->answer, "%lf", &old_min);
    else
        old_min = -99999999;

    if (parm.from2->answer)
        sscanf (parm.from2->answer, "%lf", &old_max);
    else
        old_max = 99999999;


    if (parm.count->answer)
        nlev = atoi(parm.count->answer);
    else
	nlev = -999;

    meth = mt;
    if (parm.method->answer)
    	sscanf (parm.method->answer, "%s", meth);
    else
	strcpy(meth, "area");

    if (parm.type->answer)
        typ = atoi(parm.type->answer);
    else
	typ = -999;
    
    if (parm.step->answer)
	sscanf (parm.step->answer, "%lf", &step);
    else
	step = -9999;
	
    palette=0;
    if (parm.palette->answer)
        palette = atoi(parm.palette->answer);
    
   fsname = fsn;
   if (parm.file->answer)
    	sscanf (parm.file->answer, "%s", fsname);
    else
	fsname = NULL;
	
    fsteps = NULL;	
    if ( fsname!=NULL )
    {
      fsteps = fopen (fsname, "r");
      if (NULL == fsteps)
      {
        fprintf (stderr, "%s - ", me);
        perror (fsname);
        exit(1);
      }
    }

    get_stats (old_name, mapset, &statf, flag.quiet->answer, old_min, old_max,
    nlev, meth, typ, step, palette, fsteps);

if ( fsname!=NULL ) fclose(fsteps);

exit(0);



}

