/*
 * s.out.ik -
 * developed from s.out.ascii by
 * Timur Kusherbaev 
 * SJ Geophysics Ltd. (03/04/02)
 */

#include "gis.h"
#include "site.h"

int main(int argc, char *argv[])
{
    FILE *fo;
    char *polyfile;
    int dims, strs, dbls;
    RASTER_MAP_TYPE map_type;
    int field;
    int scan_int;

    float rad;
    float catmin;
    float catmax;
    int edges;

    char *mapset, *name;
    double east, north;
    char ebuf[128], nbuf[128];
    char *desc;
    Site *site;
    char *fs;
    int all, n=0, s=0, d=0;
    RASTER_MAP_TYPE c=-1;
    char msg[200];
    FILE *fd;
    struct Cell_head window;
    struct GModule *module;
    struct
    {
	struct Flag *full, *all, *strip;
    } flag;
    struct
    {
	struct Option *input, *fs;
    } parm;

    struct Option *findex;    
    struct Option *output;

    struct Option *radius;
    struct Option *min;
    struct Option *max;
    struct Option *numedges;

    int G_trim_decimal ();

    G_gisinit (argv[0]);

    module = G_define_module();
    module->description =        
                    "Converts a GRASS site list file into an ASCII listing of "
                    "site locations and their descriptions.";
                    
    parm.input = G_define_option();
    parm.input->key = "sites";
    parm.input->type = TYPE_STRING;
    parm.input->required = YES;
    parm.input->description = "name of a sites file to be output";
    parm.input->gisprompt = "old,site_lists,sites";

    parm.fs = G_define_option();
    parm.fs->key = "fs";
    parm.fs->key_desc = "character|space|tab";
    parm.fs->type = TYPE_STRING;
    parm.fs->required = NO;
    parm.fs->description = "Output field separator";
    parm.fs->answer = "space";

    findex = G_define_option();
    findex->key = "findex";
    findex->type = TYPE_INTEGER;
    findex->required = NO;
    findex->description = "Attribute field number to use for operation";
    findex->answer = "1";

    radius = G_define_option();
    radius->key = "radius";
    radius->type = TYPE_DOUBLE;
    radius->required = NO;
    radius->description = "Radius(size) of icon (in data units)";

    numedges = G_define_option();
    numedges->key = "edges";
    numedges->type = TYPE_INTEGER;
    numedges->required = NO;
    numedges->description = "Number of icon edges ( >=3 )";

    min = G_define_option();
    min->key = "min";
    min->type = TYPE_STRING;
    min->required = NO;
    min->description = "Minimal range of data";

    max = G_define_option();
    max->key = "max";
    max->type = TYPE_DOUBLE;
    max->required = NO;
    max->description = "Maximal range of data";

    output = G_define_option();
    output->key = "output";
    output->type = TYPE_STRING;
    output->required = YES;
    output->description = "unix result file";

    flag.all = G_define_flag();
    flag.all->key = 'a';
    flag.all->description = "Output all sites (do not limit to current region)";

    if (G_parser(argc,argv))
	exit(1);
    all  = flag.all->answer;

    polyfile = output->answer;
    fo = fopen (polyfile, "w");
    if (fo == NULL)
    {
	perror (polyfile);
	exit(1);
    }

    scan_int=sscanf(findex->answer,"%d",&field);
    if ((scan_int <= 0) || field < 1)
    {
        char msg[256];
        sprintf(msg,"%s: \"%s\" is an incorrect for attr field number.\n",
        G_program_name(), findex->answer );
        G_fatal_error (msg);
        exit(1);
    }
    field--;

    if (numedges->answer)
    {
    	sscanf(numedges->answer,"%d",&edges);
    	if ((edges < 3))
    	{
     	   char msg[256];
      	  sprintf(msg,"%s: \"%s\" is an incorrect.\n",
       	 G_program_name(), numedges->answer );
        	G_fatal_error (msg);
        	exit(1);
    	}
     } else edges = 20;

    if (radius->answer)
    {
    	sscanf(radius->answer,"%f", &rad);
    	if ((rad <= 0))
    	{
     	   char msg[256];
      	  sprintf(msg,"%s: \"%s\" is an incorrect.\n",
       	 G_program_name(), radius->answer );
        	G_fatal_error (msg);
        	exit(1);
    	}
    } else rad=1;

    if (min->answer)
    {
      sscanf(min->answer,"%f",&catmin);
    } else catmin = -9999999;

    if (max->answer)
    {
    	sscanf(max->answer,"%f",&catmax);
    	if ((catmax <= catmin))
    	{
     	   char msg[256];
      	  sprintf(msg,"%s: \"%s\" is an incorrect.\n",
       	 G_program_name(), max->answer );
        	G_fatal_error (msg);
        	exit(1);
    	}
     } else catmax = 9999999;

    name = parm.input->answer;
    mapset = G_find_file ("site_lists", name, "");
    if (mapset == NULL)
    {
	sprintf (msg, "sites file [%s] not found", name);
	G_fatal_error (msg);
    }

    if (fs = parm.fs->answer)
    {
	if(strcmp (fs, "space") == 0)
	    fs = " ";
	else if(strcmp (fs, "tab") == 0)
	    fs = "\t";
    }
    else
	fs = " ";

    if (!all)
	G_get_window (&window);
    fd = G_fopen_sites_old (name, mapset);
    if (fd == NULL)
    {
	sprintf (msg, "can't open sites file [%s]", name);
	G_fatal_error (msg);
    }

    if (G_site_describe (fd, &n, &c, &s, &d)!=0)
      G_fatal_error("failed to guess format");
    site = G_site_new_struct (c, n, s, d);

    if(field >= d){								// if field dopustimo
                fprintf(stderr,"\n");
                G_fatal_error("selected decimal field column no. %d "
                        "not present in sites list.", field+1);
            }

    fprintf (fo, "%s%d\n", ".S ", edges );
    fprintf (fo, "%s%f\n", ".R ", rad );

    while (G_site_get (fd, site) == 0)
    {
      if (( site->dbl_att[field]>=catmin ) & ( site->dbl_att[field]<=catmax ))
      {
	if (all || G_site_in_region (site, &window))
	{
	      G_format_easting (site->east, ebuf, -1);
	      G_format_northing (site->north, nbuf, -1);
	      fprintf (stdout,"%s%s%s", ebuf, fs, nbuf);

	      fprintf (fo, "%s%s%s", ebuf, fs, nbuf );	

              for (n = 0; n < site->dim_alloc; ++n)
              {
                sprintf (nbuf, "%.8f", site->dim[n]);
                G_trim_decimal (nbuf);
                fprintf (stdout,"%s%s", fs, nbuf);

	        fprintf (fo, "%s%s", fs, nbuf );

              }

              fprintf (stdout,"\n");
	      fprintf (fo,"\n");
	}
      }
    }
    fclose (fd);

    fclose (fo);    

    exit(0);
}
